﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;


namespace DemoGridding
{
    public partial class Form1 : Form
    {
        // Upper limit for count of data points
        int GlMaxCount;

        public int GlnMethod;

        // Count of data points
        int GlnCount;

        // X, Y, Z values of data points
        double[] GlardX = new double[1000000];
        double[] GlardY = new double[1000000];
        double[] GlardZ = new double[1000000];

        // Values of grid
        public double[] GlardV = new double[1000000];

        // Point types of grid
        public int[] GlarnT = new int[1000000];

        // Count of rows and colums of grid
        public int GlnRowCount;
        public int GlnColumnCount;

        // The grid coordinates
        public double[] GldColumnCoord = new double[1500];
        public double[] GldRowCoord = new double[1500];

        public int lObjectHandle1;
        public int lObjectHandle2;

        int GlnPoints;
        public double[] GldLineCoordX = new double[10000];
        public double[] GldLineCoordY = new double[10000];

        // Values of contours
        public double[] GdaContourValues = new double[100];


        public Form1()
        {
            InitializeComponent();
        }

        private void Init()
        {
            DemoContour.DemoContourCreate();
            Contour.ContourCreate();
        }


        private void InitGridding()
        {

            return;
        }


        private void Form1_Load(object sender, EventArgs e)
        {
            Init();
            OpenFile("Matrix.txt");
            CreateGrid();

            CreateContours();
            DrawContours();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
             DrawContours();
        }
        private void DrawContours()
        {

            if (GlnRowCount > 50 || GlnColumnCount > 50)
                return;

            Graphics Mygraphics = pictureBox1.CreateGraphics();
            Mygraphics.Clear(Color.White);
            int longWidth = pictureBox1.Width;
            int longHeight = pictureBox1.Height;

            // Add the axis. The return code is axe index or -1 if fault
            int lBottom = DemoContour.DemoContourAxisAdd("bottom");
            int lTop = DemoContour.DemoContourAxisAdd("top");
            int lLeft = DemoContour.DemoContourAxisAdd("left");
            int lRight = DemoContour.DemoContourAxisAdd("right");

            // Change font height. The default is 18
            DemoContour.DemoContourAxisSetFontHeight(lBottom, 10);
            DemoContour.DemoContourAxisSetFontHeight(lTop, 10);
            DemoContour.DemoContourAxisSetFontHeight(lLeft, 10);
            DemoContour.DemoContourAxisSetFontHeight(lRight, 10);

            // Set the legends
            DemoContour.DemoContourAxisSetTitle(lBottom, "X, m");
            DemoContour.DemoContourAxisSetTitle(lLeft, "Y, m");
            DemoContour.DemoContourAxisSetTitle(lTop, "Top");

            // Get count of groups of contour
            int nCount = Contour.ContourIsolinesCount();

            // Set HDC for canvas
            IntPtr hdc = Mygraphics.GetHdc();
            int nRC = DemoContour.DemoContourCanvasSetHDC(hdc);

            // Set off the XY rate (1 - on, 0 - off)
            DemoContour.DemoContourCanvasSetXYRate(0);

            // Set canvas size in pixels
            nRC = DemoContour.DemoContourCanvasSetSize(longWidth, longHeight);
            
            // Set view rectangle for canvas
            // The function set pixel-coordinate system
            nRC = DemoContour.DemoContourCanvasSetLimits(GldColumnCoord[0], GldRowCoord[0], GldColumnCoord[GlnColumnCount - 1], GldRowCoord[GlnRowCount - 1]);

            string sFont = "Times New Roman"; // Use TRUE TYPE fonts ONLY
            // Default font is Arial
            nRC = DemoContour.DemoContourCanvasSetFontFaceName(sFont);
            // Default font height is 20
            nRC = DemoContour.DemoContourCanvasSetFontHeight(15);

            DemoContour.DemoContourAxisDraw();


            for (int nI = 0; nI <= nCount - 1; nI++)
            {
                // Set label value
                nRC = DemoContour.DemoContourCanvasSetLabelValue(GdaContourValues[nI]);

                // Get count of lines in nI group (for value aContourValues(nI))
                int nLines = Contour.ContourLinesCount(nI);
                for (int nJ = 0; nJ <= nLines - 1; nJ++)
                {
                    // Get count of vertexs of nJ line for nI group
                    GlnPoints = Contour.ContourPointsCount(nI, nJ);
                    nRC = Contour.ContourGetIsolinePoints(nI, nJ, out GldLineCoordX[0], out GldLineCoordY[0]);
                    // Draw contour line
                    DemoContour.DemoContourCanvasDrawContour(GlnPoints, out GldLineCoordX[0], out GldLineCoordY[0]);
                }
            }
             
            return;
        }

        private void OpenFile(string sFile)
        {
            //   Open scattered data (X, Y, Z)
            int nCount;
            StreamReader fs;
            try
            {
                fs = File.OpenText(sFile);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("File " + sFile + " is missing");
                return;
            }
            
            try
            {
                nCount = 0;
                string input = null;
                input = fs.ReadLine();
                string[] split = input.Split(new char[] {' '}, StringSplitOptions.RemoveEmptyEntries);
                GlnRowCount = Convert.ToInt32(split[0]);
                GlnColumnCount = Convert.ToInt32(split[1]);
                GlMaxCount = GlnRowCount * GlnColumnCount;
                while ((input = fs.ReadLine()) != null)
                {
                    if( nCount >= GlMaxCount )
                    {
                        System.Windows.Forms.MessageBox.Show("Too many points (" + Convert.ToString(GlMaxCount) + "is max)");
                        fs.Close();
                        return;
                    }
                    GlardV[nCount] = Convert.ToDouble(input);
                    GlarnT[nCount] = 1;
                    nCount++;
                }
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show(" line of data file is invalid");

                fs.Close();
                return;
            }
            //   Count of data points
            GlnCount = nCount;
            fs.Close();
   
            return;            
        }

        private void Form1_Resize(object sender, EventArgs e)
        {

        }


        private void CreateGrid()
        {
            //'   Create the grid
    
            double dXmin, dXmax, dYmin, dYmax, dXStep, dYStep;
            int nJ;

            dXmin = 0;
            dXmax = 1;
            dYmin = 0;
            dYmax = 1;

    
            //'   Calculate the steps of grid
            dXStep = (dXmax - dXmin) / (GlnColumnCount - 1);
            dYStep = (dYmax - dYmin) / (GlnRowCount - 1);
    
            //'   Calculate the coordinates of grid
            GldColumnCoord[0] = dXmin;
            for( nJ = 1; nJ <= GlnColumnCount - 1; nJ++ )
            {
                GldColumnCoord[nJ] = GldColumnCoord[nJ - 1] + dXStep;
            }
        
            GldRowCoord[0] = dYmin;
            for (nJ = 1; nJ <= GlnRowCount - 1; nJ++)
            {
                GldRowCoord[nJ] = GldRowCoord[nJ - 1] + dYStep;
            }
        }


        private void CreateContours()
        {
            double dMin, dMax, d;
            int nRC;
                    
            // Return codes for CONTOUR
            // CONTOUR_RC_OK = 0;
            int CONTOUR_RC_ERROR = -1;
            int CONTOUR_RC_NO_MEMORY = -2;
            //CONTOUR_RC_EXIST = 1;
            //CONTOUR_RC_NO_EXIST = 2;

    
            if( GlnRowCount > 50 || GlnColumnCount > 50 )
            {
                System.Windows.Forms.MessageBox.Show("Sorry. It is demo. Grid size no more 50x50");
                return;
            }
    
            try
            {

                nRC = Contour.ContourSetGridEx(GlnColumnCount, GlnRowCount, out GldColumnCoord[0], out GldRowCoord[0], out GlardV[0], out GlarnT[0]);
    
                if( nRC == CONTOUR_RC_ERROR || nRC == CONTOUR_RC_NO_MEMORY )
                {
                    System.Windows.Forms.MessageBox.Show("Object Contour can't create a grid");
                    return;
                }
    
                // Get the min and max value of grid field
                nRC = Contour.ContourGridGetMinMax(out dMin, out dMax);
    
                // Set contour values array ( 9 values )
                d = dMax - dMin;
                GdaContourValues[0] = dMin + d * 0.1;
                GdaContourValues[1] = dMin + d * 0.2;
                GdaContourValues[2] = dMin + d * 0.3;
                GdaContourValues[3] = dMin + d * 0.4;
                GdaContourValues[4] = dMin + d * 0.5;
                GdaContourValues[5] = dMin + d * 0.6;
                GdaContourValues[6] = dMin + d * 0.7;
                GdaContourValues[7] = dMin + d * 0.8;
                GdaContourValues[8] = dMin + d * 0.9;
                int nCount = 9;
                // Calculate 9 contours


                nRC = Contour.ContourCalculateIsolines(nCount, out GdaContourValues[0]);
                return;
             }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Function ContourCreate is missing");
                return;
            }       
        }


        private void button6_Click(object sender, EventArgs e)
        {
             CreateContours();
             DrawContours();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            DemoContour.DemoContourDelete();
            Contour.ContourDelete();
        }

    }
}
